-- ReforgeLite v1.0 by d07.RiV (Iroared)
-- All rights reserved

local function DeepCopy (t, cache)
  if type (t) ~= "table" then
    return t
  end
  local copy = {}
  for i, v in pairs (t) do
    if type (v) ~= "table" then
      copy[i] = v
    else
      cache = cache or {}
      cache[t] = copy
      if cache[v] then
        copy[i] = cache[v]
      else
        copy[i] = DeepCopy (v, lookup)
      end
    end
  end
  return copy
end

ReforgeLite = CreateFrame ("Frame", nil, UIParent)
ReforgeLite:Hide ()
ReforgeLiteDB = nil
local AddonPath = "Interface\\AddOns\\ReforgeLite\\"
local DefaultDB = {
  itemSize = 24,
  windowWidth = 800,
  windowHeight = 564,

  reforgeCheat = 5,

  convertSpirit = false,
  statWeights = {0, 0, 0, 0, 0, 0, 0, 0},
  statCaps = {
    {
      stat = 0,
      points = {
        {
          method = 1,
          value = 0,
          after = 0
        }
      }
    },
    {
      stat = 0,
      points = {
        {
          method = 1,
          value = 0,
          after = 0
        }
      }
    }
  }
}
local function MergeTables (dst, src)
  for k, v in pairs (src) do
    if type (v) ~= "table" then
      if dst[k] == nil then
        dst[k] = v
      end
    else
      if type (dst[k]) ~= "table" then
        dst[k] = {}
      end
      MergeTables (dst[k], v)
    end
  end
end

function ReforgeLite:UpgradeDBCaps (caps)
  for i = 1, #caps do
    if caps[i].points == nil then
      caps[i].points = {}
      caps[i].points[1] = {
        method = caps[i].method,
        value = caps[i].value,
        after = caps[i].after
      }
      caps[i].method = nil
      caps[i].value = nil
      caps[i].after = nil
    end
  end
end
function ReforgeLite:UpgradeDB ()
  if not ReforgeLiteDB then
    ReforgeLiteDB = DefaultDB
  else
    MergeTables (ReforgeLiteDB, DefaultDB)
  end
  self:UpgradeDBCaps (ReforgeLiteDB.statCaps)
  if ReforgeLiteDB.storedMethod then
    if ReforgeLiteDB.storedMethod.caps == nil then
      ReforgeLiteDB.storedMethod.caps = DeepCopy (ReforgeLiteDB.statCaps)
    end
    if ReforgeLiteDB.storedMethod.weights == nil then
      ReforgeLiteDB.storedMethod.weights = DeepCopy (ReforgeLiteDB.statWeights)
    end
    self:UpgradeDBCaps (ReforgeLiteDB.storedMethod.caps)
  end
  if ReforgeLiteDB.method then
    self:UpgradeDBCaps (ReforgeLiteDB.method.caps)
  end
end

local L = ReforgeLiteLocale

-----------------------------------------------------------------

ReforgeLite.itemSlots = {
  "HeadSlot",
  "NeckSlot",
  "ShoulderSlot",
  "BackSlot",
  "ChestSlot",
  "WristSlot",
  "HandsSlot",
  "WaistSlot",
  "LegsSlot",
  "FeetSlot",
  "Finger0Slot",
  "Finger1Slot",
  "Trinket0Slot",
  "Trinket1Slot",
  "MainHandSlot",
  "SecondaryHandSlot",
  "RangedSlot"
}
local function RatingStat (name_, tip_, id_)
  return {
    name = name_,
    tip = L[tip_],
    long = L[tip_ .. "Long"],
    getter = function ()
      return (GetCombatRating (id_))
    end
  }
end
ReforgeLite.itemStats = {
  {
    name = "ITEM_MOD_SPIRIT_SHORT",
    tip = L["Spirit"],
    long = L["SpiritLong"],
    getter = function ()
      return (select (2, UnitStat ("player", 5)))
    end
  },
  RatingStat ("ITEM_MOD_DODGE_RATING_SHORT", "Dodge", CR_DODGE),
  RatingStat ("ITEM_MOD_PARRY_RATING_SHORT", "Parry", CR_PARRY),
  RatingStat ("ITEM_MOD_HIT_RATING_SHORT", "Hit", CR_HIT_SPELL),
  RatingStat ("ITEM_MOD_CRIT_RATING_SHORT", "Crit", CR_CRIT_SPELL),
  RatingStat ("ITEM_MOD_HASTE_RATING_SHORT", "Haste", CR_HASTE_SPELL),
  RatingStat ("ITEM_MOD_EXPERTISE_RATING_SHORT", "Exp", CR_EXPERTISE),
  RatingStat ("ITEM_MOD_MASTERY_RATING_SHORT", "Mastery", CR_MASTERY)
}
ReforgeLite.STATS = {
  SPIRIT = 1, DODGE = 2, PARRY = 3, HIT = 4, CRIT = 5, HASTE = 6, EXP = 7, MASTERY = 8
}

ReforgeLite.REFORGE_TABLE_BASE = 112
ReforgeLite.reforgeTable = {
  {1, 2}, {1, 3}, {1, 4}, {1, 5}, {1, 6}, {1, 7}, {1, 8},
  {2, 1}, {2, 3}, {2, 4}, {2, 5}, {2, 6}, {2, 7}, {2, 8},
  {3, 1}, {3, 2}, {3, 4}, {3, 5}, {3, 6}, {3, 7}, {3, 8},
  {4, 1}, {4, 2}, {4, 3}, {4, 5}, {4, 6}, {4, 7}, {4, 8},
  {5, 1}, {5, 2}, {5, 3}, {5, 4}, {5, 6}, {5, 7}, {5, 8},
  {6, 1}, {6, 2}, {6, 3}, {6, 4}, {6, 5}, {6, 7}, {6, 8},
  {7, 1}, {7, 2}, {7, 3}, {7, 4}, {7, 5}, {7, 6}, {7, 8},
  {8, 1}, {8, 2}, {8, 3}, {8, 4}, {8, 5}, {8, 6}, {8, 7},
}

function ReforgeLite:UpdateWindowSize ()
  self.db.windowWidth = self:GetWidth ()
  self.db.windowHeight = self:GetHeight ()
end

function ReforgeLite:GetCapScore (weights, cap, value)
  local score = 0
  for i = #cap.points, 1, -1 do
    if value > cap.points[i].value then
      score = score + cap.points[i].after * (value - cap.points[i].value)
      value = cap.points[i].value
    end
  end
  score = score + weights[cap.stat] * value
  return score
end
function ReforgeLite:GetStatScore (weights, caps, stat, value)
  if stat == caps[1].stat then
    return self:GetCapScore (weights, caps[1], value)
  elseif stat == caps[2].stat then
    return self:GetCapScore (weights, caps[2], value)
  else
    return weights[stat] * value
  end
end

-------------------------------------------------------------------------

GUI = {
  widgetCount = 0,
  GenerateWidgetName = function (gui)
    gui.widgetCount = gui.widgetCount + 1
    return "ReforgeLiteWidget" .. gui.widgetCount
  end,

  SetTooltip = function (gui, widget, tip)
    widget:SetScript ("OnEnter", function (self)
      GameTooltip:SetOwner (self, "ANCHORLEFT")
      GameTooltip:SetText (tip)
      GameTooltip:Show ()
    end)
    widget:SetScript ("OnLeave", function ()
      GameTooltip:Hide ()
    end)
  end,

  editBoxes = {},
  CreateEditBox = function (gui, parent, width, height, default, setter)
    local box
    if #gui.editBoxes > 0 then
      box = table.remove (gui.editBoxes, 1)
      box:SetParent (parent)
      box:Show ()
    else
      box = CreateFrame ("EditBox", gui:GenerateWidgetName (), parent, "InputBoxTemplate")
      box:SetAutoFocus (false)
      box:SetFontObject (ChatFontNormal)
      box:SetNumeric ()
      box:SetTextInsets (0, 0, 3, 3)
      box:SetMaxLetters (8)
      box:SetScript ("OnEnterPressed", function (self)
        self:ClearFocus ()
      end)
      box.Recycle = function (self)
        self:Hide ()
        self:SetScript ("OnEditFocusLost", nil)
        self:SetScript ("OnEnter", nil)
        self:SetScript ("OnLeave", nil)
        table.insert (gui.editBoxes, self)
      end
    end
    if width then
      box:SetWidth (width)
    end
    if height then
      box:SetHeight (height)
    end
    box:SetText (default)
    if setter then
      box:SetScript ("OnEditFocusLost", function (self)
        setter (tonumber (self:GetText ()) or 0)
      end)
    end
    return box
  end,

  dropdowns = {},
  CreateDropdown = function (gui, parent, values, default, setter, width)
    local sel
    if #gui.dropdowns > 0 then
      sel = table.remove (gui.dropdowns, 1)
      sel:SetParent (parent)
      sel:Show ()
    else
      local name = gui:GenerateWidgetName ()
      sel = CreateFrame ("Frame", name, parent, "UIDropDownMenuTemplate")
      sel.Initialize = function (self)
        local info = UIDropDownMenu_CreateInfo ()
        for i, v in ipairs (self.values) do
          info.text = v.name
          info.func = function (inf)
            UIDropDownMenu_SetSelectedValue (self, inf.value)
            self.value = inf.value
            if self.setter then self.setter (inf.value) end
          end
          info.value = v.value
          info.checked = (self.value == v.value)
          UIDropDownMenu_AddButton (info)
        end
      end
      sel:SetScript ("OnShow", function (self)
        UIDropDownMenu_Initialize (self, self.Initialize)
        UIDropDownMenu_SetSelectedValue (self, self.value)
      end)
      UIDropDownMenu_JustifyText (sel, "LEFT")
      sel:SetHeight (50)
      _G[name .. "Left"]:SetHeight (50)
      _G[name .. "Middle"]:SetHeight (50)
      _G[name .. "Right"]:SetHeight (50)
      _G[name .. "Button"]:SetPoint ("TOPRIGHT", _G[name .. "Right"], "TOPRIGHT", -16, -12)
      sel.Recycle = function (self)
        self:Hide ()
        self:SetScript ("OnEnter", nil)
        self:SetScript ("OnLeave", nil)
        sel.setter = nil
        table.insert (gui.dropdowns, self)
      end
    end
    sel.value = default
    sel.values = values
    sel.setter = setter
    UIDropDownMenu_Initialize (sel, sel.Initialize)
    UIDropDownMenu_SetSelectedValue (sel, sel.value)
    if width then
      UIDropDownMenu_SetWidth (sel, width)
    end
    return sel
  end,

  checkButtons = {},
  CreateCheckButton = function (gui, parent, text, default, setter)
    local btn
    if #gui.checkButtons > 0 then
      btn = table.remove (gui.checkButtons, 1)
      btn:SetParent (parent)
      btn:Show ()
    else
      local name = gui:GenerateWidgetName ()
      btn = CreateFrame ("CheckButton", name, parent, "UICheckButtonTemplate")
      btn.Recycle = function (self)
        self:Hide ()
        self:SetScript ("OnEnter", nil)
        self:SetScript ("OnLeave", nil)
        self.SetScript ("OnClick", nil)
        table.insert (gui.checkButtons, self)
      end
    end
    _G[btn:GetName () .. "Text"]:SetText (text)
    btn:SetChecked (default)
    if setter then
      btn:SetScript ("OnClick", function (self)
        setter (self:GetChecked ())
      end)
    end
    return btn
  end,

  imgButtons = {},
  CreateImageButton = function (gui, parent, width, height, img, pus, hlt, handler)
    local btn
    if #gui.imgButtons > 0 then
      btn = table.remove (gui.imgButtons, 1)
      btn:SetParent (parent)
      btn:Show ()
    else
      btn = CreateFrame ("Button", nil, parent)
      btn.Recycle = function (self)
        self:Hide ()
        self:SetScript ("OnEnter", nil)
        self:SetScript ("OnLeave", nil)
        self.SetScript ("OnClick", nil)
        table.insert (gui.imgButtons, self)
      end
    end
    btn:SetNormalTexture (img)
    btn:SetPushedTexture (pus)
    btn:SetHighlightTexture (hlt)
    btn:SetWidth (width)
    btn:SetHeight (height)
    if handler then
      btn:SetScript ("OnClick", handler)
    end
    return btn
  end
}

-------------------------------------------------------------------------

function ReforgeLite:CreateHLine (x1, x2, y, w, color, parent)
  local line = (parent or self.content):CreateTexture (nil, "ARTWORK")
  line:SetDrawLayer ("ARTWORK")
  line:SetTexture (color[1], color[2], color[3], color[4])
  if x1 > x2 then
    x1, x2 = x2, x1
  end
  line:ClearAllPoints ()
  line:SetTexCoord (0, 0, 0, 1, 1, 0, 1, 1)
  line.width = w
  line:SetPoint ("BOTTOMLEFT", parent, "TOPLEFT", x1, y - w / 2)
  line:SetPoint ("TOPRIGHT", parent, "TOPLEFT", x2, y + w / 2)
  line:Show ()
  line.SetPos = function (self, x1, x2, y)
    if x1 > x2 then
      x1, x2 = x2, x1
    end
    self:ClearAllPoints ()
    self:SetPoint ("BOTTOMLEFT", self:GetParent (), "TOPLEFT", x1, y - self.width / 2)
    self:SetPoint ("TOPRIGHT", self:GetParent (), "TOPLEFT", x2, y + self.width / 2)
  end
  return line
end

function ReforgeLite:CreateVLine (x, y1, y2, w, color, parent)
  local line = (parent or self.content):CreateTexture (nil, "ARTWORK")
  line:SetDrawLayer ("ARTWORK")
  line:SetTexture (color[1], color[2], color[3], color[4])
  if y1 > y2 then
    y1, y2 = y2, y1
  end
  line:ClearAllPoints ()
  line:SetTexCoord (1, 0, 0, 0, 1, 1, 0, 1)
  line.width = w
  line:SetPoint ("BOTTOMLEFT", self, "TOPLEFT", x - w / 2, y1)
  line:SetPoint ("TOPRIGHT", self, "TOPLEFT", x + w / 2, y2)
  line:Show ()
  line.SetPos = function (self, x, y1, y2)
    if y1 > y2 then
      y1, y2 = y2, y1
    end
    self:ClearAllPoints ()
    self:SetPoint ("BOTTOMLEFT", self:GetParent (), "TOPLEFT", x - self.width / 2, y1)
    self:SetPoint ("TOPRIGHT", self:GetParent (), "TOPLEFT", x + self.width / 2, y2)
  end
  return line
end

function ReforgeLite:CreateTable (rows, cols, firstRow, firstColumn, gridColor, parent)
  parent = parent or self.content
  firstRow = firstRow or 0
  firstColumn = firstColumn or 0

  local t = CreateFrame ("Frame", nil, parent)
  t:ClearAllPoints ()
  t:SetWidth (400)
  t:SetHeight (400)
  t:SetPoint ("TOPLEFT")

  t.rows = rows
  t.cols = cols
  t.gridColor = gridColor
  t.rowPos = {}
  t.colPos = {}
  t.rowPos[-1] = 0
  t.rowPos[0] = firstRow
  t.colPos[-1] = 0
  t.colPos[0] = firstColumn

  t.SetRowHeight = function (self, n, h)
    if h then
      if n < 0 or n > self.rows then
        return
      end
      self.autoRowHeight = false
      local delta = h - self.rowPos[n] + self.rowPos[n - 1]
      for i = n, self.rows do
        self.rowPos[i] = self.rowPos[i] + delta
      end
      self:SetHeight (self.rowPos[self.rows])
      if n == 0 and self.hlines then
        if h == 0 then
          self.hlines[-1]:Hide ()
        else
          self.hlines[-1]:Show ()
        end
      end
    elseif n == "AUTO" then
      self.autoRowHeight = true
      local height = (self:GetHeight () - self.rowPos[0]) / self.rows
      for i = 1, self.rows do
        self.rowPos[i] = self.rowPos[i - 1] + height
      end
    else
      self.autoRowHeight = false
      for i = 1, self.rows do
        self.rowPos[i] = self.rowPos[i - 1] + n
      end
      self:SetHeight (self.rowPos[self.rows])
    end
  end
  t.SetColumnWidth = function (self, n, w)
    if w then
      if n < 0 or n > self.cols then
        return
      end
      self.autoColWidth = false
      local delta = w - self.colPos[n] + self.colPos[n - 1]
      for i = n, self.cols do
        self.colPos[i] = self.colPos[i] + delta
      end
      self:SetWidth (self.colPos[self.cols])
      if n == 0 and self.vlines then
        if w == 0 then
          self.vlines[-1]:Hide ()
        else
          self.vlines[-1]:Show ()
        end
      end
    elseif n == "AUTO" then
      self.autoColWidth = true
      local width = (self:GetWidth () - self.colPos[0]) / self.cols
      for i = 1, self.cols do
        self.colPos[i] = self.colPos[i - 1] + width
      end
    else
      self.autoColWidth = false
      for i = 1, self.cols do
        self.colPos[i] = self.colPos[i - 1] + n
      end
      self:SetWidth (self.colPos[self.cols])
    end
  end
  t.AddRow = function (self, i, n)
    i = i or (self.rows + 1)
    n = n or 1
    local height = (i == self.rows + 1) and (self.rowPos[i - 1] - self.rowPos[i - 2]) or (self.rowPos[i] - self.rowPos[i - 1])
    for r = self.rows, i, -1 do
      self.cells[r + n] = self.cells[r]
      self.rowPos[r + n] = self.rowPos[r] + n * height
    end
    for r = 1, n do
      self.cells[i + r - 1] = {}
      self.rowPos[i + r - 1] = self.rowPos[i + r - 2] + height
      self.hlines[self.rows + r] = ReforgeLite:CreateHLine (0, 0, 0, self.gridColor, self)
    end
    self.rows = self.rows + n
    self:OnUpdateFix ()
  end
  t.MoveRow = function (self, i, to)
    local h = self.rowPos[i] - self.rowPos[i - 1]
    local cells = self.cells[i]
    if to > i then
      for r = i + 1, to do
        self.cells[r - 1] = self.cells[r]
        self.rowPos[r - 1] = self.rowPos[r] - h
      end
    elseif to < i then
      for r = i - 1, to, -1 do
        self.cells[r + 1] = self.cells[r]
        self.rowPos[r + 1] = self.rowPos[r] + h
      end
    end
    self.cells[to] = cells
    self.rowPos[to] = self.rowPos[to - 1] + h
    self:OnUpdateFix ()
  end
  t.DeleteRow = function (self, i)
    local height = self.rowPos[i] - self.rowPos[i - 1]
    for j = 0, self.cols do
      if self.cells[i][j] then
        if self.cells[i][j].Recycle then
          self.cells[i][j]:Recycle ()
        else
          self.cells[i][j]:Hide ()
        end
      end
    end
    for r = i + 1, self.rows do
      self.cells[r - 1] = self.cells[r]
      self.rowPos[r - 1] = self.rowPos[r] - height
    end
    self.rows = self.rows - 1
    self:OnUpdateFix ()
  end

  t:SetRowHeight ("AUTO")
  t:SetColumnWidth ("AUTO")

  t.GetCellY = function (self, i)
    local n = math.ceil (i)
    if n < 0 then n = 0 end
    if n > self.rows then n = self.rows end
    return - (self.rowPos[n] + (self.rowPos[n - 1] - self.rowPos[n]) * (n - i))
  end
  t.GetCellX = function (self, j)
    local n = math.ceil (j)
    if n < 0 then n = 0 end
    if n > self.cols then n = self.cols end
    return self.colPos[n] + (self.colPos[n - 1] - self.colPos[n]) * (n - j)
  end
  t.AlignCell = function (self, i, j)
    local x = self.cells[i][j].offsX
    local y = self.cells[i][j].offsY
    if self.cells[i][j].align == "TOPLEFT" then
      self.cells[i][j]:SetPoint ("TOPLEFT", self, "TOPLEFT", self:GetCellX (j - 1) + 2 + x, self:GetCellY (i - 1) - 2 + y)
    elseif self.cells[i][j].align == "LEFT" then
      self.cells[i][j]:SetPoint ("LEFT", self, "TOPLEFT", self:GetCellX (j - 1) + 2 + x, self:GetCellY (i - 0.5) + y)
    elseif self.cells[i][j].align == "BOTTOMLEFT" then
      self.cells[i][j]:SetPoint ("BOTTOMLEFT", self, "TOPLEFT", self:GetCellX (j - 1) + 2 + x, self:GetCellY (i) + 2 + y)

    elseif self.cells[i][j].align == "TOP" then
      self.cells[i][j]:SetPoint ("TOP", self, "TOPLEFT", self:GetCellX (j - 0.5) + x, self:GetCellY (j - 1) - 2 + y)
    elseif self.cells[i][j].align == "CENTER" then
      self.cells[i][j]:SetPoint ("CENTER", self, "TOPLEFT", self:GetCellX (j - 0.5) + x, self:GetCellY (i - 0.5) + y)
    elseif self.cells[i][j].align == "BOTTOM" then
      self.cells[i][j]:SetPoint ("BOTTOM", self, "TOPLEFT", self:GetCellX (j - 0.5) + x, self:GetCellY (j) + 2 + y)

    elseif self.cells[i][j].align == "TOPRIGHT" then
      self.cells[i][j]:SetPoint ("TOPRIGHT", self, "TOPLEFT", self:GetCellX (j) - 2 + x, self:GetCellY (i - 1) - 2 + y)
    elseif self.cells[i][j].align == "RIGHT" then
      self.cells[i][j]:SetPoint ("RIGHT", self, "TOPLEFT", self:GetCellX (j) - 2 + x, self:GetCellY (i - 0.5) + y)
    elseif self.cells[i][j].align == "BOTTOMRIGHT" then
      self.cells[i][j]:SetPoint ("BOTTOMRIGHT", self, "TOPLEFT", self:GetCellX (j) - 2 + x, self:GetCellY (i) + 2 + y)
    end
  end
  t.OnUpdateFix = function (self)
    self:SetScript ("OnUpdate", nil)

    if self.autoRowHeight then
      local height = (self:GetHeight () - self.rowPos[0]) / self.rows
      for i = 1, self.rows do
        self.rowPos[i] = self.rowPos[i - 1] + height
      end
    end
    if self.autoColWidth then
      local width = (self:GetWidth () - self.colPos[0]) / self.cols
      for i = 1, self.cols do
        self.colPos[i] = self.colPos[i - 1] + width
      end
    end

    if self.gridColor then
      for i = -1, self.rows do
        t.hlines[i]:SetPos (0, self.colPos[self.cols], -self.rowPos[i])
      end
      for i = -1, self.cols do
        t.vlines[i]:SetPos (self.colPos[i], 0, -self.rowPos[self.rows])
      end
    end
    for i = -1, self.rows do
      for j = -1, self.cols do
        if self.cells[i][j] then
          self:AlignCell (i, j)
        end
      end
    end
  end
  t:SetScript ("OnSizeChanged", function (self)
    self:SetScript ("OnUpdate", self.OnUpdateFix)
  end)

  if gridColor then
    t.hlines = {}
    t.vlines = {}
    for i = -1, rows do
      t.hlines[i] = self:CreateHLine (0, t.colPos[t.cols], -t.rowPos[i], 1, gridColor, t)
    end
    for i = -1, cols do
      t.vlines[i] = self:CreateVLine (t.colPos[i], 0, -t.rowPos[t.rows], 1, gridColor, t)
    end
    if firstRow == 0 then
      t.hlines[-1]:Hide ()
    end
    if firstColumn == 0 then
      t.vlines[-1]:Hide ()
    end
  end
  t.cells = {}
  for i = -1, rows do
    t.cells[i] = {}
  end

  t.SetCell = function (self, i, j, value, align, offsX, offsY)
    align = align or "CENTER"
    offsX = offsX or 0
    offsY = offsY or 0
    self.cells[i][j] = value
    self.cells[i][j].align = align
    self.cells[i][j].offsX = offsX
    self.cells[i][j].offsY = offsY
    self:AlignCell (i, j)
  end
  t.SetCellText = function (self, i, j, text, align, color, font)
    align = align or "CENTER"
    color = color or {1, 1, 1}
    font = font or "GameFontNormalSmall"
    offsX = offsX or 0
    offsY = offsY or 0

    self.cells[i][j] = self:CreateFontString (nil, "OVERLAY", font)
    self.cells[i][j]:SetTextColor (color[1], color[2], color[3])
    self.cells[i][j]:SetText (text)
    self.cells[i][j].align = align
    self.cells[i][j].offsX = offsX
    self.cells[i][j].offsY = offsY
    self:AlignCell (i, j)
  end
  
  return t
end

------------------------------------------------------------------------

function ReforgeLite:CreateCategory (name)
  local c = CreateFrame ("Frame", nil, self.content)
  c:ClearAllPoints ()
  c:SetWidth (16)
  c:SetHeight (16)
  c.expanded = true

  c.name = c:CreateFontString (nil, "OVERLAY", "GameFontNormal")
  c.catname = c.name
  c.name:SetPoint ("TOPLEFT", c, "TOPLEFT", 18, 0)
  c.name:SetTextColor (1, 1, 1)
  c.name:SetText (name)

  c.button = CreateFrame ("Button", nil, c)
  c.button:ClearAllPoints ()
  c.button:SetWidth (14)
  c.button:SetHeight (14)
  c.button:SetPoint ("TOPLEFT", c, "TOPLEFT", 0, 0)
  c.button:SetHighlightTexture ("Interface\\Buttons\\UI-PlusButton-Hilight")
  c.button.UpdateTexture = function (self)
    if self:GetParent ().expanded then
      self:SetNormalTexture ("Interface\\Buttons\\UI-MinusButton-Up")
      self:SetPushedTexture ("Interface\\Buttons\\UI-MinusButton-Down")
    else
      self:SetNormalTexture ("Interface\\Buttons\\UI-PlusButton-Up")
      self:SetPushedTexture ("Interface\\Buttons\\UI-PlusButton-Down")
    end
  end
  c.button:UpdateTexture ()
  c.button:SetScript ("OnClick", function (self)
    self:GetParent ():Toggle ()
  end)
  c.button.anchor = {point = "TOPLEFT", rel = c, relPoint = "TOPLEFT", x = 0, y = 0}

  c.frames = {}
  c.anchors = {}
  c.AddFrame = function (self, frame)
    table.insert (self.frames, frame)
    frame.category = self
  end

  c.Toggle = function (self)
    self.expanded = not self.expanded
    if c.expanded then
      for k, v in pairs (self.frames) do
        v:Show ()
      end
      for k, v in pairs (self.anchors) do
        v.frame:SetPoint (v.point, v.rel, v.relPoint, v.x, v.y)
      end
    else
      for k, v in pairs (self.frames) do
        v:Hide ()
      end
      for k, v in pairs (self.anchors) do
        v.frame:SetPoint (v.point, self.button, v.relPoint, v.x, v.y)
      end
    end
    self.button:UpdateTexture ()
    ReforgeLite:UpdateContentSize ()
  end

  return c
end

function ReforgeLite:SetAnchor (frame_, point_, rel_, relPoint_, offsX, offsY)
  if rel_ and rel_.catname and rel_.button then
    rel_ = rel_.button
  end
  if rel_.category then
    table.insert (rel_.category.anchors, {frame = frame_, point = point_, rel = rel_, relPoint = relPoint_, x = offsX, y = offsY})
    if rel_.category.expanded then
      frame_:SetPoint (point_, rel_, relPoint_, offsX, offsY)
    else
      frame_:SetPoint (point_, rel_.category.button, relPoint_, offsX, offsY)
    end
  else
    frame_:SetPoint (point_, rel_, relPoint_, offsX, offsY)
  end
  frame_.anchor = {point = point_, rel = rel_, relPoint = relPoint_, x = offsX, y = offsY}
end
function ReforgeLite:GetFrameY (frame)
  local cur = frame
  local offs = 0
  while cur and cur ~= self.content do
    if cur.anchor == nil then
      return offs
    end
    if cur.anchor.point:find ("BOTTOM") then
      offs = offs + cur:GetHeight ()
    end
    local rel = cur.anchor.rel
    if rel.category and not rel.category.expanded then
      rel = rel.category.button
    end
    if cur.anchor.relPoint:find ("BOTTOM") then
      offs = offs - rel:GetHeight ()
    end
    offs = offs + cur.anchor.y
    cur = rel
  end
  return offs
end

local function SetTextDelta (text, value, cur, override)
  override = override or (value - cur)
  if override == 0 then
    text:SetTextColor (0.7, 0.7, 0.7)
  elseif override > 0 then
    text:SetTextColor (0.6, 1, 0.6)
  else
    text:SetTextColor (1, 0.4, 0.4)
  end
  text:SetText (string.format ("%+d", value - cur))
end

------------------------------------------------------------------------

function ReforgeLite:SetScroll (value)
  local viewheight = self.scrollFrame:GetHeight ()
  local height = self.content:GetHeight ()
  local offset

  if viewheight > height then
    offset = 0
  else
    offset = math.floor ((height - viewheight) / 1000 * value)
  end
  self.content:ClearAllPoints ()
  self.content:SetPoint ("TOPLEFT", 0, offset)
  self.content:SetPoint ("TOPRIGHT", 0, offset)
  self.scrollOffset = offset
  self.scrollValue = value
end
function ReforgeLite:MoveScroll (value)
  local viewheight = self.scrollFrame:GetHeight ()
  local height = self.content:GetHeight ()
  if self.scrollBarShown then
    local diff = height - viewheight
    local delta = (value > 0 and -1 or 1)
    self.scrollBar:SetValue (min (max (self.scrollValue + delta * (1000 / (diff / 45)), 0), 1000))
  end
end
function ReforgeLite:FixScroll (value)
  self:SetScript ("OnUpdate", nil)

  local offset = self.scrollOffset
  local viewheight = self.scrollFrame:GetHeight ()
  local height = self.content:GetHeight ()
  if height < viewheight + 2 then
    if self.scrollBarShown then
      self.scrollBarShown = false
      self.scrollBar:Hide ()
      self.scrollBar:SetValue (0)
    end
  else
    if not self.scrollBarShown then
      self.scrollBarShown = true
      self.scrollBar:Show ()
    end
    local value = (offset / (height - viewheight) * 1000)
    if value > 1000 then value = 1000 end
    self.scrollBar:SetValue (value)
    self:SetScroll (value)
    if value < 1000 then
      self.content:ClearAllPoints ()
      self.content:SetPoint ("TOPLEFT", 0, offset)
      self.content:SetPoint ("TOPRIGHT", 0, offset)
    end
  end
end

function ReforgeLite:CreateFrame (title, width, height)
  local title = "Reforge Lite"
  self:SetFrameStrata ("DIALOG")
  self:ClearAllPoints ()
  self:SetWidth (self.db.windowWidth)
  self:SetHeight (self.db.windowHeight)
  self:SetMinResize (780, 500)
  self:SetMaxResize (1000, 800)
  if self.db.windowX and self.db.windowY then
    self:SetPoint ("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.windowX, self.db.windowY)
  else
    self:SetPoint ("CENTER")
  end
  self:SetBackdrop ({
    bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
    edgeFile = AddonPath .. "textures\\frameborder", edgeSize = 32,
    insets = {left = 1, right = 1, top = 20, bottom = 1}
  })
  self:SetBackdropBorderColor (0.5, 0.5, 0.5)
  self:SetBackdropColor (0.1, 0.1, 0.1)

  self:EnableMouse (true)
  self:SetMovable (true)
  self:SetResizable (true)
  self:SetScript ("OnMouseDown", function (self, arg)
    if self.methodWindow and self:GetFrameLevel () < self.methodWindow:GetFrameLevel () then
      self:SetFrameLevel (self.methodWindow:GetFrameLevel () + 10)
    end
    if arg == "LeftButton" then
      self:StartMoving ()
      self.moving = true
    end
  end)
  self:SetScript ("OnMouseUp", function (self)
    if self.moving then
      self:StopMovingOrSizing ()
      self.moving = false
      self.db.windowX = self:GetLeft ()
      self.db.windowY = self:GetTop ()
    end
  end)

  self.title = self:CreateFontString (nil, "OVERLAY", "GameFontNormal")
  self.title:SetPoint ("TOPLEFT", self, "TOPLEFT", 6, -15)
  self.title:SetTextColor (1, 1, 1)
  self.title:SetText (title)

  self.close = CreateFrame ("Button", nil, self)
  self.close:SetNormalTexture ("Interface\\Buttons\\UI-Panel-MinimizeButton-Up.blp")
  self.close:SetPushedTexture ("Interface\\Buttons\\UI-Panel-MinimizeButton-Down.blp")
  self.close:SetHighlightTexture ("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight.blp")
  self.close:SetWidth (20)
  self.close:SetHeight (20)
  self.close:SetPoint ("TOPRIGHT", self, "TOPRIGHT", -4, -12)
  self.close:SetScript ("OnClick", function (self)
    self:GetParent ():Hide ()
  end)

  self.leftGrip = CreateFrame ("Button", nil, self)
  self.leftGrip:SetNormalTexture (AddonPath .. "textures\\leftgrip")
  self.leftGrip:SetHighlightTexture (AddonPath .. "textures\\leftgrip")
  self.leftGrip:SetWidth (20)
  self.leftGrip:SetHeight (20)
  self.leftGrip:SetPoint ("BOTTOMLEFT", self, "BOTTOMLEFT", -4, -4)
  self.leftGrip:SetScript ("OnMouseDown", function (self, arg)
    if arg == "LeftButton" then
      self:GetParent ():StartSizing ("BOTTOMLEFT")
      self:GetParent ().sizing = true
    end
  end)
  self.leftGrip:SetScript ("OnMouseUp", function (self)
    if self:GetParent ().sizing then
      self:GetParent ():StopMovingOrSizing ()
      self:GetParent ().sizing = false
      self:GetParent ():UpdateWindowSize ()
    end
  end)

  self.rightGrip = CreateFrame ("Button", nil, self)
  self.rightGrip:SetNormalTexture (AddonPath .. "textures\\rightGrip")
  self.rightGrip:SetHighlightTexture (AddonPath .. "textures\\rightGrip")
  self.rightGrip:SetWidth (20)
  self.rightGrip:SetHeight (20)
  self.rightGrip:SetPoint ("BOTTOMRIGHT", self, "BOTTOMRIGHT", 4, -4)
  self.rightGrip:SetScript ("OnMouseDown", function (self, arg)
    if arg == "LeftButton" then
      self:GetParent ():StartSizing ("BOTTOMRIGHT")
      self:GetParent ().sizing = true
    end
  end)
  self.rightGrip:SetScript ("OnMouseUp", function (self)
    if self:GetParent ().sizing then
      self:GetParent ():StopMovingOrSizing ()
      self:GetParent ().sizing = false
      self:GetParent ():UpdateWindowSize ()
    end
  end)

  self:CreateItemTable ()

  self.scrollValue = 0
  self.scrollOffset = 0
  self.scrollBarShown = false

  self.scrollFrame = CreateFrame ("ScrollFrame", nil, self)
  self.scrollFrame:ClearAllPoints ()
  self.scrollFrame:SetPoint ("LEFT", self.itemTable, "RIGHT", 10, 0)
  self.scrollFrame:SetPoint ("TOP", self, "TOP", 0, -40)
  self.scrollFrame:SetPoint ("BOTTOMRIGHT", self, "BOTTOMRIGHT", -22, 15)
  self.scrollFrame:EnableMouseWheel (true)
  self.scrollFrame:SetScript ("OnMouseWheel", function (self, value)
    ReforgeLite:MoveScroll (value)
  end)
  self.scrollFrame:SetScript ("OnSizeChanged", function (self)
    ReforgeLite:SetScript ("OnUpdate", ReforgeLite.FixScroll)
  end)

  self.scrollBar = CreateFrame ("Slider", "ReforgeLiteScrollBar", self.scrollFrame, "UIPanelScrollBarTemplate")
  self.scrollBar:SetPoint ("TOPLEFT", self.scrollFrame, "TOPRIGHT", 4, -16)
  self.scrollBar:SetPoint ("BOTTOMLEFT", self.scrollFrame, "BOTTOMRIGHT", 4, 16)
  self.scrollBar:SetMinMaxValues (0, 1000)
  self.scrollBar:SetValueStep (1)
  self.scrollBar:SetValue (0)
  self.scrollBar:SetWidth (16)
  self.scrollBar:SetScript ("OnValueChanged", function (self, value)
    ReforgeLite:SetScroll (value)
  end)
  self.scrollBar:Hide ()

  self.scrollBg = self.scrollBar:CreateTexture (nil, "BACKGROUND")
  self.scrollBg:SetAllPoints (self.scrollBar)
  self.scrollBg:SetTexture (0, 0, 0, 0.4)

  self.content = CreateFrame ("Frame", nil, self.scrollFrame)
  self.scrollFrame:SetScrollChild (self.content)
  self.content:ClearAllPoints ()
  self.content:SetPoint ("TOPLEFT")
  self.content:SetPoint ("TOPRIGHT")
  self.content:SetHeight (1000)

  self:CreateOptionList ()

  self:SetScript ("OnUpdate", self.FixScroll)
end

function ReforgeLite:CreateItemTable ()
  self.itemLevel = self:CreateFontString (nil, "OVERLAY", "GameFontNormal")
  self.itemLevel:SetPoint ("TOPLEFT", self, "TOPLEFT", 12, -40)
  self.itemLevel:SetTextColor (1, 1, 0.8)
  self.itemLevel:SetText ("Item level: 0")

  self.itemTable = self:CreateTable (#self.itemSlots + 1, #self.itemStats, self.db.itemSize, self.db.itemSize + 4, {0.5, 0.5, 0.5, 1}, self)
  self.itemTable:SetPoint ("TOPLEFT", self.itemLevel, "BOTTOMLEFT", 0, -10)
  self.itemTable:SetPoint ("BOTTOM", self, "BOTTOM", 0, 10)
  self.itemTable:SetWidth (400)

  for i, v in ipairs (self.itemStats) do
    self.itemTable:SetCellText (0, i, v.tip)
  end
  self.itemData = {}
  for i, v in ipairs (self.itemSlots) do
    self.itemData[i] = CreateFrame ("Frame", nil, self.itemTable)
    self.itemData[i].slot = v
    self.itemData[i]:ClearAllPoints ()
    self.itemData[i]:SetWidth (self.db.itemSize)
    self.itemData[i]:SetHeight (self.db.itemSize)
    self.itemTable:SetCell (i, 0, self.itemData[i])
    self.itemData[i]:EnableMouse (true)
    self.itemData[i]:SetScript ("OnEnter", function (self)
      GameTooltip:SetOwner (self, "ANCHORLEFT")
      local hasItem, hasCooldown, repairCost = GameTooltip:SetInventoryItem ("player", self.slotId)
      if not hasItem then
        local text = _G[strupper (self.slot)]
        if self.checkRelic then
          text = _G["RELICSLOT"]
        end
        GameTooltip:SetText (text)
      end
      GameTooltip:Show ()
    end)
    self.itemData[i]:SetScript ("OnLeave", function (self)
      GameTooltip:Hide ()
    end)
    self.itemData[i].slotId, self.itemData[i].slotTexture, self.itemData[i].checkRelic = GetInventorySlotInfo (v)
    self.itemData[i].checkRelic = self.itemData[i].checkRelic and UnitHasRelicSlot ("player")
    if self.itemData[i].checkRelic then
      self.itemData[i].slotTexture = "Interface\\Paperdoll\\UI-PaperDoll-Slot-Relic.blp"
    end
    self.itemData[i].texture = self.itemData[i]:CreateTexture (nil, "OVERLAY")
    self.itemData[i].texture:SetAllPoints (self.itemData[i])
    self.itemData[i].texture:SetTexture (self.itemData[i].slotTexture)

    self.itemData[i].stats = {}
    for j, s in ipairs (self.itemStats) do
      self.itemData[i].stats[j] = self.itemTable:CreateFontString (nil, "OVERLAY", "GameFontNormalSmall")
      self.itemTable:SetCell (i, j, self.itemData[i].stats[j])
      self.itemData[i].stats[j]:SetTextColor (0.8, 0.8, 0.8)
      self.itemData[i].stats[j]:SetText ("-")
    end
  end
  self.statTotals = {}
  self.itemTable:SetCellText (#self.itemSlots + 1, 0, L["Sum"], "CENTER", {1, 0.8, 0})
  for i, v in ipairs (self.itemStats) do
    self.statTotals[i] = self.itemTable:CreateFontString (nil, "OVERLAY", "GameFontNormalSmall")
    self.itemTable:SetCell (#self.itemSlots + 1, i, self.statTotals[i])
    self.statTotals[i]:SetTextColor (1, 0.8, 0)
    self.statTotals[i]:SetText ("0")
  end
end

function ReforgeLite:AddCapPoint (i, loading)
  local row = (loading or #self.db.caps[i].points + 1) + (i == 1 and 1 or #self.db.caps[1].points + 2)
  local point = #self.db.caps[i].points + 1
  self.statCaps:AddRow (row)

  local rem = GUI:CreateImageButton (self.statCaps, 20, 20, "Interface\\Buttons\\UI-MinusButton-Up",
    "Interface\\Buttons\\UI-MinusButton-Down", "Interface\\Buttons\\UI-PlusButton-Hilight", function ()
    self:RemoveCapPoint (i, point)
  end)
  local methodList = {{value = 1, name = L["at least"]}, {value = 2, name = L["at most"]}, {value = 3, name = "-"}}
  local method = GUI:CreateDropdown (self.statCaps, methodList, 1,
    function (val) self.db.statCaps[i].points[point].method = val end, 80)
  local value = GUI:CreateEditBox (self.statCaps, 50, 30, 0, function (val)
    self.db.statCaps[i].points[point].value = val
    self:ReorderCapPoint (i, point)
  end)
  local after = GUI:CreateEditBox (self.statCaps, 50, 30, 0, function (val) self.db.statCaps[i].points[point].after = val end)

  GUI:SetTooltip (rem, L["Remove cap"])
  GUI:SetTooltip (value, L["Cap value"])
  GUI:SetTooltip (after, L["Weight after cap"])

  self.statCaps:SetCell (row, 0, rem)
  self.statCaps:SetCell (row, 1, method)
  self.statCaps:SetCell (row, 2, value)
  self.statCaps:SetCell (row, 3, after)

  if not loading then
    table.insert (self.db.statCaps[i].points, 1, {value = 0, method = 1, after = 0})
    self:UpdateCapPoints (i)
  end
end
function ReforgeLite:RemoveCapPoint (i, point)
  local row = #self.db.statCaps[1].points + (i == 1 and 1 or #self.db.statCaps[2].points + 2)
  self.statCaps:DeleteRow (row)
  table.remove (self.db.statCaps[i].points, point)
  self:UpdateCapPoints (i)
end
function ReforgeLite:ReorderCapPoint (i, point)
  local newpos = point
  while newpos > 1 and self.db.statCaps[i].points[newpos - 1].value > self.db.statCaps[i].points[point].value do
    newpos = newpos - 1
  end
  while newpos < #self.db.statCaps[i].points and self.db.statCaps[i].points[newpos + 1].value < self.db.statCaps[i].points[point].value do
    newpos = newpos + 1
  end
  if newpos ~= point then
    local tmp = self.db.statCaps[i].points[point]
    table.remove (self.db.statCaps[i].points, point)
    table.insert (self.db.statCaps[i].points, newpos, tmp)
    self:UpdateCapPoints (i)
  end
end
function ReforgeLite:UpdateCapPoints (i)
  local base = (i == 1 and 1 or #self.db.statCaps[1].points + 2)
  for point = 1, #self.db.statCaps[i].points do
    self.statCaps.cells[base + point][1]:SetSelectedValue (self.db.statCaps[i].points[point].method)
    _G[self.statCaps.cells[base + point][2]:GetName () .. "Text"]:SetText (self.db.statCaps[i].points[point].value)
    _G[self.statCaps.cells[base + point][3]:GetName () .. "Text"]:SetText (self.db.statCaps[i].points[point].after)
  end
end
function ReforgeLite:CreateOptionList ()
  self.statWeightsCategory = self:CreateCategory (L["Stat weights"])
  self:SetAnchor (self.statWeightsCategory, "TOPLEFT", self.content, "TOPLEFT", 2, -2)

  self.statWeights = self:CreateTable (#self.itemStats, 1, 0, 100, nil)
  self.statWeightsCategory:AddFrame (self.statWeights)
  self:SetAnchor (self.statWeights, "TOPLEFT", self.statWeightsCategory, "BOTTOMLEFT", 0, -5)
  self.statWeights:SetRowHeight (self.db.itemSize + 2)
  self.statWeights:SetColumnWidth (80)
  for i, v in ipairs (self.itemStats) do
    self.statWeights:SetCellText (i, 0, v.long, "LEFT")

    self.statWeights[i] = GUI:CreateEditBox (self.statWeights, 60, self.db.itemSize,
      self.db.statWeights[i], function (val) self.db.statWeights[i] = val end)
    self.statWeights:SetCell (i, 1, self.statWeights[i])
  end

  self.convertSpirit = GUI:CreateCheckButton (self.content, L["Spirit to hit"],
    self.db.spiritToHit, function (val) self.db.spiritToHit = val end)
  self.statWeightsCategory:AddFrame (self.convertSpirit)
  self:SetAnchor (self.convertSpirit, "TOPLEFT", self.statWeights, "TOPRIGHT", 10, 0)

  self.statCaps = self:CreateTable (2, 3, nil, self.db.itemSize + 2)
  self.statWeightsCategory:AddFrame (self.statCaps)
  self:SetAnchor (self.statCaps, "TOPLEFT", self.statWeights, "BOTTOMLEFT", 0, -10)
  self.statCaps:SetRowHeight (self.db.itemSize + 2)
  self.statCaps:SetColumnWidth (1, 100)
  self.statCaps:SetColumnWidth (2, 60)
  self.statCaps:SetColumnWidth (3, 60)
  local statList = {{value = 0, name = L["None"]}}
  for i, v in ipairs (self.itemStats) do
    table.insert (statList, {value = i, name = v.long})
  end
  for i = 1, 2 do
    self.statCaps[i] = {}
    self.statCaps[i].stat = GUI:CreateDropdown (self.statCaps, statList, self.db.statCaps[i].stat,
      function (val) self.db.statCaps[i].stat = val end, 100)
    self.statCaps[i].add = GUI:CreateImageButton (self.statCaps, 20, 20, "Interface\\Buttons\\UI-PlusButton-Up",
      "Interface\\Buttons\\UI-PlusButton-Down", "Interface\\Buttons\\UI-PlusButton-Hilight", function ()
      self:AddCapPoint (i)
    end)
    GUI:SetTooltip (self.statCaps[i].add, L["Add cap"])
    self.statCaps:SetCell (i, 0, self.statCaps[i].stat, "LEFT", 10, -10)
    self.statCaps:SetCell (i, 2, self.statCaps[i].add, "LEFT")
  end
  for i = 1, 2 do
    for point = 1, #self.db.statCaps[i].points do
      self:AddCapPoint (i, point)
    end
  end

  self.computeButton = CreateFrame ("Button", "ReforgeLiteConfirmButton", self.content, "UIPanelButtonTemplate")
  self.computeButton:SetWidth (114)
  self.computeButton:SetHeight (22)
  self.computeButton:SetText (L["Compute"])
  self.computeButton:SetScript ("OnClick", function (self)
    ReforgeLite.method = ReforgeLite:Compute ()
    ReforgeLite:UpdateMethodCategory ()
  end)
  self:SetAnchor (self.computeButton, "TOPLEFT", self.statCaps, "BOTTOMLEFT", 0, -10)

  self.quality = CreateFrame ("Slider", nil, self.content)
  self:SetAnchor (self.quality, "LEFT", self.computeButton, "RIGHT", 15, 0)
  self.quality:SetOrientation ("HORIZONTAL")
  self.quality:SetWidth (150)
  self.quality:SetHeight (15)
  self.quality:SetHitRectInsets (0, 0, -10, 0)
  self.quality:SetBackdrop ({bgFile = "Interface\\Buttons\\UI-SliderBar-Background",
    edgeFile = "Interface\\Buttons\\UI-SliderBar-Border", tile = true, tileSize = 8, edgeSize = 8,
    insets = {left = 3, right = 3, top = 6, bottom = 6}
  })
  self.quality:SetThumbTexture ("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")
  self.quality:SetMinMaxValues (1, 20)
  self.quality:SetValueStep (1)
  self.quality:SetValue (self.db.reforgeCheat)
  self.quality:EnableMouseWheel (false)
  self.quality:SetScript ("OnValueChanged", function (self)
    ReforgeLite.db.reforgeCheat = self:GetValue ()
  end)

  self.quality.label = self.quality:CreateFontString (nil, "OVERLAY", "GameFontNormal")
  self.quality.label:SetPoint ("BOTTOM", self.quality, "TOP", 0, 0)
  self.quality.label:SetTextColor (1, 1, 1)
  self.quality.label:SetText (L["Speed"])
  self.quality.lowtext = self.quality:CreateFontString (nil, "ARTWORK", "GameFontHighlightSmall")
  self.quality.lowtext:SetPoint ("TOPLEFT", self.quality, "BOTTOMLEFT", 2, 3)
  self.quality.lowtext:SetText ("1")
  self.quality.hightext = self.quality:CreateFontString (nil, "ARTWORK", "GameFontHighlightSmall")
  self.quality.hightext:SetPoint ("TOPRIGHT", self.quality, "BOTTOMRIGHT", -2, 3)
  self.quality.hightext:SetText ("20")

  self.storedCategory = self:CreateCategory (L["Best result"])
  self:SetAnchor (self.storedCategory, "TOPLEFT", self.computeButton, "BOTTOMLEFT", 0, -10)
  self.storedScore = self.content:CreateFontString (nil, "OVERLAY", "GameFontNormal")
  self.storedCategory:AddFrame (self.storedScore)
  self:SetAnchor (self.storedScore, "TOPLEFT", self.storedCategory, "BOTTOMLEFT", 0, -8)
  self.storedScore:SetTextColor (1, 1, 1)
  self.storedScore:SetText (L["Score"] .. ":")
  self.storedScore.score = self.content:CreateFontString (nil, "OVERLAY", "GameFontNormal")
  self.storedCategory:AddFrame (self.storedScore.score)
  self:SetAnchor (self.storedScore.score, "BOTTOMLEFT", self.storedScore, "BOTTOMRIGHT", 8, 0)
  self.storedScore.score:SetTextColor (0.7, 0.7, 0.7)
  self.storedScore.score:SetText ("+0")

  self.storedClear = CreateFrame ("Button", "ReforgeLiteStoredClear", self.content, "UIPanelButtonTemplate")
  self.storedClear:SetWidth (114)
  self.storedClear:SetHeight (22)
  self.storedClear:SetText (L["Clear"])
  self.storedClear:SetScript ("OnClick", function (self)
    ReforgeLite:ClearStoredMethod ()
  end)
  self.storedCategory:AddFrame (self.storedClear)
  self:SetAnchor (self.storedClear, "TOPLEFT", self.storedScore, "BOTTOMLEFT", 0, -8)

  self.storedRestore = CreateFrame ("Button", "ReforgeLiteStoredRestore", self.content, "UIPanelButtonTemplate")
  self.storedRestore:SetWidth (114)
  self.storedRestore:SetHeight (22)
  self.storedRestore:SetText (L["Restore"])
  self.storedRestore:SetScript ("OnClick", function (self)
    ReforgeLite:RestoreStoredMethod ()
  end)
  self.storedCategory:AddFrame (self.storedRestore)
  self:SetAnchor (self.storedRestore, "BOTTOMLEFT", self.storedClear, "BOTTOMRIGHT", 8, 0)

  if self.db.storedMethod then
    SetTextDelta (self.storedScore.score, self.db.storedMethod.score, self:GetCurrentScore ())
    self.storedClear:Enable ()
    self.storedRestore:Enable ()
  else
    self:ClearStoredMethod ()
  end

  self.lastElement = CreateFrame ("Frame", nil, self.content)
  self.lastElement:ClearAllPoints ()
  self.lastElement:SetWidth (0)
  self.lastElement:SetHeight (0)
  self:SetAnchor (self.lastElement, "TOPLEFT", self.storedClear, "BOTTOMLEFT", 0, -10)
  self:UpdateContentSize ()

  if self.db.method then
    ReforgeLite:UpdateMethodCategory ()
  end
end
function ReforgeLite:GetCurrentScore ()
  local score = 0
  for i = 1, #self.itemStats do
    local value = self.itemStats[i].getter ()
    score = score + self:GetStatScore (self.db.weights, self.db.statCaps, i, value)
  end
  return score
end
function ReforgeLite:UpdateMethodCategory ()
  if self.methodCategory == nil then
    self.methodCategory = self:CreateCategory (L["Result"])
    self:SetAnchor (self.methodCategory, "TOPLEFT", self.computeButton, "BOTTOMLEFT", 0, -10)

    self.methodStats = self:CreateTable (#self.itemStats, 2, self.db.itemSize, 60, {0.5, 0.5, 0.5, 1})
    self.methodCategory:AddFrame (self.methodStats)
    self:SetAnchor (self.methodStats, "TOPLEFT", self.methodCategory, "BOTTOMLEFT", 0, -5)
    self.methodStats:SetRowHeight (self.db.itemSize + 2)
    self.methodStats:SetColumnWidth (60)

    self.methodStats:SetCellText (0, 0, L["Score"], "LEFT", {1, 0.8, 0})
    self.methodStats.score = self.methodStats:CreateFontString (nil, "OVERLAY", "GameFontNormalSmall")
    self.methodStats:SetCell (0, 1, self.methodStats.score)
    self.methodStats.score:SetTextColor (1, 0.8, 0)
    self.methodStats.score:SetText ("0")
    self.methodStats.scoreDelta = self.methodStats:CreateFontString (nil, "OVERLAY", "GameFontNormalSmall")
    self.methodStats:SetCell (0, 2, self.methodStats.scoreDelta)
    self.methodStats.scoreDelta:SetTextColor (0.7, 0.7, 0.7)
    self.methodStats.scoreDelta:SetText ("+0")

    for i, v in ipairs (self.itemStats) do
      self.methodStats:SetCellText (i, 0, v.tip, "LEFT")

      self.methodStats[i] = {}

      self.methodStats[i].value = self.methodStats:CreateFontString (nil, "OVERLAY", "GameFontNormalSmall")
      self.methodStats:SetCell (i, 1, self.methodStats[i].value)
      self.methodStats[i].value:SetTextColor (1, 1, 1)
      self.methodStats[i].value:SetText ("0")

      self.methodStats[i].delta = self.methodStats:CreateFontString (nil, "OVERLAY", "GameFontNormalSmall")
      self.methodStats:SetCell (i, 2, self.methodStats[i].delta)
      self.methodStats[i].delta:SetTextColor (0.7, 0.7, 0.7)
      self.methodStats[i].delta:SetText ("+0")
    end

    self.methodShow = CreateFrame ("Button", "ReforgeLiteMethodShowButton", self.content, "UIPanelButtonTemplate")
    self.methodShow:SetWidth (114)
    self.methodShow:SetHeight (22)
    self.methodShow:SetText (L["Show"])
    self.methodShow:SetScript ("OnClick", function (self)
      ReforgeLite:ShowMethodWindow ()
    end)
    self.methodCategory:AddFrame (self.methodShow)
    self:SetAnchor (self.methodShow, "TOPLEFT", self.methodStats, "BOTTOMLEFT", 0, -5)

    self:SetAnchor (self.storedCategory, "TOPLEFT", self.methodShow, "BOTTOMLEFT", 0, -10)
  end

  if self.db.storedMethod == nil or self.db.method.score > self.db.storedMethod.score then
    self.db.storedMethod = self.db.method
    SetTextDelta (self.storedScore.score, self.db.storedMethod.score, self:GetCurrentScore ())
    self.storedClear:Enable ()
    self.storedRestore:Enable ()
  end

  self.methodStats.score:SetText (self.db.method.score)
  SetTextDelta (self.methodStats.scoreDelta, self.db.method.score, self:GetCurrentScore ())
  for i = 1, #self.itemStats do
    self.methodStats[i].value:SetText (self.db.method.stats[i])
    local override = nil
    local value = self.itemStats[i].getter ()
    if self:GetStatScore (self.db.method.weights, self.db.method.caps, i, self.db.method.stats[i]) == self:GetStatScore (self.db.method.weights, self.db.method.caps, i, value) then
      override = 0
    end
    SetTextDelta (self.methodStats[i].delta, self.db.method.stats[i], value, override)
  end

  self:UpdateContentSize ()
end
function ReforgeLite:ClearStoredMethod ()
  self.db.storedMethod = nil
  self.storedScore.score:SetTextColor (0.7, 0.7, 0.7)
  self.storedScore.score:SetText (L["None"])
  self.storedClear:Disable ()
  self.storedRestore:Disable ()
end
function ReforgeLite:RestoreStoredMethod ()
  if self.db.storedMethod then
    self.db.method = self.db.storedMethod
    self:UpdateMethodCategory ()
  end
end
function ReforgeLite:UpdateContentSize ()
  self.content:SetHeight (-self:GetFrameY (self.lastElement))
  self:SetScript ("OnUpdate", self.FixScroll)
end

function ReforgeLite:GetReforgeID (item)
  local id = tonumber (item:match ("item:%d+:%d+:%d+:%d+:%d+:%d+:%-?%d+:%-?%d+:%d+:(%d+)"))
  return (id ~= 0 and (id - self.REFORGE_TABLE_BASE) or nil)
end

function GetUnitItemLevel (unit)
  local itemLevel = 0
  local slots = {}
  for i, v in ipairs (ReforgeLite.itemData) do
    local item = GetInventoryItemLink (unit, v.slotId)
    local iLevel, equipSlot = 0, nil
    if item then
      _, _, _, iLevel, _, _, _, _, equipSlot = GetItemInfo (item)
    end
    slots[i] = {level = iLevel, slot = equipSlot}
    itemLevel = itemLevel + iLevel
  end
  if slots[16].slot == nil and slots[15].slot == "INVTYPE_2HWEAPON" then
    itemLevel = itemLevel + slots[15].level
  end
  return itemLevel / 17
end

function ReforgeLite:UpdateItems ()
  local itemLevel = GetUnitItemLevel ("player")
  for i, v in ipairs (self.itemData) do
    local item = GetInventoryItemLink ("player", v.slotId)
    local texture = GetInventoryItemTexture ("player", v.slotId)
    local stats = {}
    local reforgeSrc, reforgeDst = nil, nil
    if texture then
      v.item = item
      v.texture:SetTexture (texture)
      stats = GetItemStats (item)
      local reforge = self:GetReforgeID (item)
      if reforge then
        reforgeSrc, reforgeDst = self.itemStats[self.reforgeTable[reforge][1]].name, self.itemStats[self.reforgeTable[reforge][2]].name
        local amount = math.floor (stats[reforgeSrc] * 0.4)
        stats[reforgeSrc] = stats[reforgeSrc] - amount
        stats[reforgeDst] = (stats[reforgeDst] or 0) + amount
      end
    else
      v.item = nil
      v.texture:SetTexture (v.slotTexture)
    end
    for j, s in ipairs (self.itemStats) do
      if stats[s.name] and stats[s.name] ~= 0 then
        self.itemData[i].stats[j]:SetText (stats[s.name])
        if s.name == reforgeSrc then
          self.itemData[i].stats[j]:SetTextColor (1, 0.4, 0.4)
        elseif s.name == reforgeDst then
          self.itemData[i].stats[j]:SetTextColor (0.6, 1, 0.6)
        else
          self.itemData[i].stats[j]:SetTextColor (1, 1, 1)
        end
      else
        self.itemData[i].stats[j]:SetText ("-")
        self.itemData[i].stats[j]:SetTextColor (0.8, 0.8, 0.8)
      end
    end
  end
  for i, v in ipairs (self.itemStats) do
    self.statTotals[i]:SetText (v.getter ())
  end
  itemLevel = math.floor (itemLevel + 0.5)
  self.itemLevel:SetText ("Item level: " .. itemLevel)

  if self.methodStats and self.db.method then
    if self.db.storedMethod then
      SetTextDelta (self.storedScore.score, self.db.storedMethod.score, self:GetCurrentScore ())
    end
    SetTextDelta (self.methodStats.scoreDelta, self.db.method.score, self:GetCurrentScore ())
    for i = 1, #self.itemStats do
      local value = self.itemStats[i].getter ()
      local override = nil
      if self:GetStatScore (self.db.method.weights, self.db.method.caps, i, self.db.method.stats[i]) == self:GetStatScore (self.db.method.weights, self.db.method.caps, i, value) then
        override = 0
      end
      SetTextDelta (self.methodStats[i].delta, self.db.method.stats[i], value, override)
    end
  end
end
function ReforgeLite:QueueUpdate ()
  self:SetScript ("OnUpdate", function (self)
    self:SetScript ("OnUpdate", nil)
    self:UpdateItems ()
  end)
  if self.methodWindow then
    self.methodWindow:SetScript ("OnUpdate", function (self)
      self:SetScript ("OnUpdate", nil)
      ReforgeLite:UpdateMethodChecks ()
    end)
  end
end

--------------------------------------------------------------------------

function ReforgeLite:ShowMethodWindow ()
  if self.methodWindow == nil then
    self.methodWindow = CreateFrame ("Frame", nil, UIParent)
    self.methodWindow:SetFrameStrata ("DIALOG")
    self.methodWindow:SetFrameLevel (ReforgeLite:GetFrameLevel () + 10)
    self.methodWindow:ClearAllPoints ()
    self.methodWindow:SetWidth (300)
    self.methodWindow:SetHeight (520)
    if self.db.methodWindowX and self.db.methodWindowY then
      self.methodWindow:SetPoint ("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.methodWindowX, self.db.methodWindowY)
    else
      self.methodWindow:SetPoint ("CENTER")
    end
    self.methodWindow:SetBackdrop ({
      bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
      edgeFile = AddonPath .. "textures\\frameborder", edgeSize = 32,
      insets = {left = 1, right = 1, top = 20, bottom = 1}
    })
    self.methodWindow:SetBackdropBorderColor (0.0, 0.5, 0.0)
    self.methodWindow:SetBackdropColor (0.1, 0.1, 0.1)

    self.methodWindow:EnableMouse (true)
    self.methodWindow:SetMovable (true)
    self.methodWindow:SetScript ("OnMouseDown", function (self, arg)
      if self:GetFrameLevel () < ReforgeLite:GetFrameLevel () then
        self:SetFrameLevel (ReforgeLite:GetFrameLevel () + 10)
      end
      if arg == "LeftButton" then
        self:StartMoving ()
        self.moving = true
      end
    end)
    self.methodWindow:SetScript ("OnMouseUp", function (self)
      if self.moving then
        self:StopMovingOrSizing ()
        self.moving = false
        ReforgeLite.db.methodWindowX = self:GetLeft ()
        ReforgeLite.db.methodWindowY = self:GetTop ()
      end
    end)

    self.methodWindow.title = self.methodWindow:CreateFontString (nil, "OVERLAY", "GameFontNormal")
    self.methodWindow.title:SetPoint ("TOPLEFT", self.methodWindow, "TOPLEFT", 6, -15)
    self.methodWindow.title:SetTextColor (1, 1, 1)
    self.methodWindow.title:SetText ("ReforgeLite Output")

    self.methodWindow.close = CreateFrame ("Button", nil, self.methodWindow)
    self.methodWindow.close:SetNormalTexture ("Interface\\Buttons\\UI-Panel-MinimizeButton-Up.blp")
    self.methodWindow.close:SetPushedTexture ("Interface\\Buttons\\UI-Panel-MinimizeButton-Down.blp")
    self.methodWindow.close:SetHighlightTexture ("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight.blp")
    self.methodWindow.close:SetWidth (20)
    self.methodWindow.close:SetHeight (20)
    self.methodWindow.close:SetPoint ("TOPRIGHT", self.methodWindow, "TOPRIGHT", -4, -12)
    self.methodWindow.close:SetScript ("OnClick", function (self)
      self:GetParent ():Hide ()
    end)
    
    self.methodWindow.itemTable = self:CreateTable (#self.itemSlots + 1, 3, 0, 0, nil, self.methodWindow)
    self.methodWindow:ClearAllPoints ()
    self.methodWindow.itemTable:SetPoint ("TOPLEFT", self.methodWindow, "TOPLEFT", 12, -40)
    self.methodWindow.itemTable:SetRowHeight (26)
    self.methodWindow.itemTable:SetColumnWidth (1, self.db.itemSize)
    self.methodWindow.itemTable:SetColumnWidth (2, self.db.itemSize + 2)
    self.methodWindow.itemTable:SetColumnWidth (3, 274 - self.db.itemSize * 2)

    self.methodWindow.items = {}
    for i, v in ipairs (self.itemSlots) do
      self.methodWindow.items[i] = CreateFrame ("Frame", nil, self.methodWindow.itemTable)
      self.methodWindow.items[i].slot = v
      self.methodWindow.items[i]:ClearAllPoints ()
      self.methodWindow.items[i]:SetWidth (self.db.itemSize)
      self.methodWindow.items[i]:SetHeight (self.db.itemSize)
      self.methodWindow.itemTable:SetCell (i, 2, self.methodWindow.items[i])
      self.methodWindow.items[i]:EnableMouse (true)
      self.methodWindow.items[i]:SetScript ("OnEnter", function (self)
        GameTooltip:SetOwner (self, "ANCHORLEFT")
        if self.item then
          GameTooltip:SetHyperlink (self.item)
        else
          local text = _G[strupper (self.slot)]
          if self.checkRelic then
            text = _G["RELICSLOT"]
          end
          GameTooltip:SetText (text)
        end
        GameTooltip:Show ()
      end)
      self.methodWindow.items[i]:SetScript ("OnLeave", function (self)
        GameTooltip:Hide ()
      end)
      self.methodWindow.items[i].slotId, self.methodWindow.items[i].slotTexture, self.methodWindow.items[i].checkRelic = GetInventorySlotInfo (v)
      self.methodWindow.items[i].checkRelic = self.methodWindow.items[i].checkRelic and UnitHasRelicSlot ("player")
      if self.methodWindow.items[i].checkRelic then
        self.methodWindow.items[i].slotTexture = "Interface\\Paperdoll\\UI-PaperDoll-Slot-Relic.blp"
      end
      self.methodWindow.items[i].texture = self.methodWindow.items[i]:CreateTexture (nil, "OVERLAY")
      self.methodWindow.items[i].texture:SetAllPoints (self.methodWindow.items[i])
      self.methodWindow.items[i].texture:SetTexture (self.methodWindow.items[i].slotTexture)

      self.methodWindow.items[i].reforge = self.methodWindow.itemTable:CreateFontString (nil, "OVERLAY", "GameFontNormal")
      self.methodWindow.itemTable:SetCell (i, 3, self.methodWindow.items[i].reforge, "LEFT")
      self.methodWindow.items[i].reforge:SetTextColor (1, 1, 1)
      self.methodWindow.items[i].reforge:SetText ("")

      self.methodWindow.items[i].check = self.methodWindow.itemTable:CreateTexture (nil, "OVERLAY")
      self.methodWindow.items[i].check:SetWidth (self.db.itemSize)
      self.methodWindow.items[i].check:SetHeight (self.db.itemSize)
      self.methodWindow.itemTable:SetCell (i, 1, self.methodWindow.items[i].check)
      self.methodWindow.items[i].check:SetTexture ("Interface\\Buttons\\UI-CheckBox-Check")
      self.methodWindow.items[i].check:Hide ()
    end
    self.methodWindow.reforge = CreateFrame ("Button", "ReforgeLiteReforgeButton", self.methodWindow, "UIPanelButtonTemplate")
    self.methodWindow.reforge:SetWidth (114)
    self.methodWindow.reforge:SetHeight (22)
    self.methodWindow.reforge:SetPoint ("BOTTOMLEFT", self.methodWindow, "BOTTOMLEFT", 12, 12)
    self.methodWindow.reforge:SetText (L["Reforge"])
    self.methodWindow.reforge:SetScript ("OnClick", function (self)
      ReforgeLite:DoReforge ()
    end)

    self.methodWindow.costTip = self.methodWindow:CreateFontString (nil, "OVERLAY", "GameFontNormal")
    self.methodWindow.costTip:SetPoint ("BOTTOMLEFT", self.methodWindow.reforge, "BOTTOMRIGHT", 8, 0)
    self.methodWindow.costTip:SetTextColor (1, 1, 1)
    self.methodWindow.costTip:SetText (L["Cost"] .. ":")
    self.methodWindow.cost = CreateFrame ("Frame", "ReforgeLiteReforgeCost", self.methodWindow, "SmallMoneyFrameTemplate")
    MoneyFrame_SetType (self.methodWindow.cost, "STATIC")
    self.methodWindow.cost:SetPoint ("BOTTOMLEFT", self.methodWindow.costTip, "BOTTOMRIGHT", 5, 0)
  end

  for i, v in ipairs (self.methodWindow.items) do
    local item = GetInventoryItemLink ("player", v.slotId)
    local texture = GetInventoryItemTexture ("player", v.slotId)
    if texture then
      v.item = item
      v.texture:SetTexture (texture)
    else
      v.item = nil
      v.texture:SetTexture (v.slotTexture)
    end
    if self.db.method.items[i].reforge then
      v.reforge:SetText (string.format ("%d %s > %s", self.db.method.items[i].amount,
        self.itemStats[self.db.method.items[i].src].long, self.itemStats[self.db.method.items[i].dst].long))
      v.reforge:SetTextColor (1, 1, 1)
    else
      v.reforge:SetText (L["No reforge"])
      v.reforge:SetTextColor (0.7, 0.7, 0.7)
    end
  end
  self:UpdateMethodChecks ()
  self.methodWindow:Show ()
end
function ReforgeLite:UpdateMethodChecks ()
  if self.methodWindow and self.db.method then
    local cost = 0
    local anyDiffer = false
    for i, v in ipairs (self.methodWindow.items) do
      local item = GetInventoryItemLink ("player", v.slotId)
      v.item = item
      local texture = GetInventoryItemTexture ("player", v.slotId)
      v.texture:SetTexture (texture or v.slotTexture)
      if self.db.method.items[i].reforge == (item and self:GetReforgeID (item)) then
        v.check:Show ()
      else
        if item then
          anyDiffer = true
        end
        v.check:Hide ()
        if self.db.method.items[i].reforge then
          cost = cost + (item and select (11, GetItemInfo (item)) or 0)
        end
      end
    end
    if anyDiffer then
      if ReforgingFrame and ReforgingFrame:IsShown () then
        self.methodWindow.reforge:Enable ()
      else
        self.methodWindow.reforge:Disable ()
      end
      self.methodWindow.costTip:Show ()
      self.methodWindow.cost:Show ()
    else
      self.methodWindow.reforge:Disable ()
      self.methodWindow.costTip:Hide ()
      self.methodWindow.cost:Hide ()
    end
    MoneyFrame_Update (self.methodWindow.cost, cost)
  end
end

--------------------------------------------------------------------------

function ReforgeLite:DoReforgeUpdate ()
  if self.db.method and self.methodWindow.reforge:IsShown () and ReforgingFrame and ReforgingFrame:IsShown () then
    while self.curReforgeItem <= #self.methodWindow.items do
      local i = self.curReforgeItem
      if i ~= 0 then
        local slot = self.methodWindow.items[i].slotId
        local item = GetInventoryItemLink ("player", slot)
        if item and self.db.method.items[i].reforge ~= self:GetReforgeID (item) then
          ClearCursor ()
          SetReforgeFromCursorItem ()
          ClearCursor ()
          PickupInventoryItem (slot)
          SetReforgeFromCursorItem ()
          if self:GetReforgeID (item) then
            ReforgeItem (0)
          elseif self.db.method.items[i].reforge then
            local id = 0
            local stats = GetItemStats (item)
            for s = 1, #self.reforgeTable do
              if (stats[self.itemStats[self.reforgeTable[s][1]].name] or 0) ~= 0 and (stats[self.itemStats[self.reforgeTable[s][2]].name] or 0) == 0 then
                id = id + 1
              end
              if self.reforgeTable[s][1] == self.db.method.items[i].src and self.reforgeTable[s][2] == self.db.method.items[i].dst then
                ReforgeItem (id)
                return
              end
            end
            self.methodWindow.reforge:SetScript ("OnUpdate", nil)
          end
          return
        end
      end
      self.curReforgeItem = i + 1
    end
  end
  self.methodWindow.reforge:SetScript ("OnUpdate", nil)
end
function ReforgeLite:DoReforge ()
  if self.db.method and self.methodWindow and ReforgingFrame and ReforgingFrame:IsShown () then
    self.curReforgeItem = 0
    self.methodWindow.reforge:SetScript ("OnUpdate", function (self) ReforgeLite:DoReforgeUpdate () end)
  end
end

--------------------------------------------------------------------------

function ReforgeLite:OnEvent (event, ...)
  if self[event] then
    self[event] (self, ...)
  end
  if event == "COMBAT_RATING_UPDATE" or event == "MASTERY_UPDATE" or event == "PLAYER_EQUIPMENT_CHANGED" then
    self:QueueUpdate ()
  end
  if event == "FORGE_MASTER_OPENED" or event == "FORGE_MASTER_CLOSED" then
    self:QueueUpdate ()
  end
end

function ReforgeLite:ADDON_LOADED (addon)
  if addon == "ReforgeLite" then
    self:UpgradeDB ()
    self.db = ReforgeLiteDB

    self:CreateFrame ()

    self:RegisterEvent ("COMBAT_RATING_UPDATE")
    self:RegisterEvent ("MASTERY_UPDATE")
    self:RegisterEvent ("PLAYER_EQUIPMENT_CHANGED")
    self:RegisterEvent ("FORGE_MASTER_OPENED")
    self:RegisterEvent ("FORGE_MASTER_CLOSED")

    SlashCmdList["ReforgeLite"] = function (cmd) ReforgeLite:OnCommand (cmd) end
    SLASH_ReforgeLite1 = "/reforge"
  end
end

ReforgeLite:SetScript ("OnEvent", ReforgeLite.OnEvent)
ReforgeLite:RegisterEvent ("ADDON_LOADED")

function ReforgeLite:OnCommand (cmd)
  self:UpdateItems ()
  self:Show ()
end
